#pragma once
#include <atl/string.h>

namespace cpptesk
{
namespace ts
{

/// @brief Metafunction to construct signature of <tt>init</tt> method.
///
/// @tparam S scenario class
/// @return type signature of the <tt>init</tt> method.
template<typename S>
struct InitMethod
{
    typedef bool (S::*type)(int argc, char** argv);
};
/// @brief Metafunction to construct signature of <tt>finish</tt> method.
///
/// @tparam S scenario class
/// @return signature type of the <tt>finish</tt> method.
template<typename S>
struct FinishMethod
{
    typedef void (S::*type)();
};

/// @brief Metafunction to construct signature of <tt>get_state</tt> method.
///
/// @tparam S scenario class
/// @tparam T return value of the <tt>get_state</tt> method.
/// @return type signature of the <tt>get_state</tt> method.
template<typename S, typename T>
struct GetStateMethod
{
    typedef T (S::*type)();
};

/// @brief Metafunction to construct signature of <tt>get_state_id</tt> method.
///
/// @tparam S scenario class
/// @return signature type of the <tt>get_state_id</tt> method.
template<typename S>
struct GetStateIdMethod
{
    typedef String* (S::*type)(Object*);
};

/// @brief Metafunction to construct signature of <tt>is_stationary_state</tt> method.
///
/// @tparam S scenario class
/// @return signature type of the <tt>is_stationary_state</tt> method.
template<typename S>
struct IsStationaryStateMethod
{
    typedef bool (S::*type)();
};

/// @brief Metafunction to construct signature of <tt>observe_model_state</tt> method.
///
/// @tparam S scenario class
/// @return signature type of the <tt>observe_model_state</tt> method.
template<typename S>
struct ObserveModelStateMethod
{
    typedef void (S::*type)();
};

/// @brief Metafunction to construct signature of <tt>save_model_state</tt> method.
///
/// @tparam S scenario class
/// @tparam T return value of the <tt>save_model_state</tt> method.
/// @return type signature of the <tt>save_model_state</tt> method.
template<typename S, typename T>
struct SaveModelStateMethod
{
    typedef T (S::*type)();
};

/// @brief Metafunction to construct signature of <tt>restore_model_state</tt> method.
///
/// @tparam S scenario class
/// @return type signature of the <tt>restore_model_state</tt> method.
template<typename S>
struct RestoreModelStateMethod
{
    typedef void (S::*type)(Object*);
};

} // namespace cpptesk::ts
} // namespace cpptesk
