/** @file fifo_scen.cpp
 *   FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_scen.h>

#include <tracer/tracer.hpp>

namespace cpptesk {
namespace examples {
namespace fifo {

bool FIFOScenario::init(int argc, char ** argv)
{
    return true;
}

void FIFOScenario::finish()
{
    //  :   run  0.
    dut.finalize();
}

bool FIFOScenario::scen_nop(Ctx& ctx)
{
    //    (    26 ):
    // int& param1 = IVAR(a);
    // int& param2 = IVAR(b);

    //        IBEGIN

    //      IBEGIN
    IBEGIN
    //  IBEGIN   
    // for(param1 = 0; param1 < 2; param1++) {
    //    for(param2 = 0; param2 < 2; param2++) {

    //     IACTION    .
    IACTION
    {
        //  
        dut.refresh_coverage();
        //     1 .
        dut.cycle();

        //     .
        YIELD(dut.verdict());
    }
    //     }
    // }
    IEND
}

bool FIFOScenario::scen_push(Ctx& ctx)
{
    IBEGIN
    IACTION
    {
        //   :         .
        if(dut.isIface1Ready() && !dut.is_full()) {
            //   ,          .
            //        .
            InputData msg = InputData();
            dut.start(&FIFO::push_msg, dut.iface1, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

bool FIFOScenario::scen_pop(cpptesk::ts::IntAbcCtx& ctx)
{
    IBEGIN
    IACTION
    {
        //   :         .
        if(dut.isIface2Ready() && !dut.is_empty()) {
            //   ,          .
            //        .
            InputData msg = InputData();
            dut.start(&FIFO::pop_msg, dut.iface2, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

std::string FIFOScenario::get_state_fifo()
{
    std::string state;
    std::stringstream out;

    out << "{fifo:" << dut.fifo_fulness() << ", write_started:" << dut.isIface1Ready() << ", read_started:" << dut.isIface2Ready() << "}";
    state = out.str();
#ifdef DEBUG
    std::cout << "get_state_fifo(): " << state << std::endl;
#endif
    return state;
}

FIFOScenario::FIFOScenario(void) : ScenarioBase()
{
}
FIFOScenario::~FIFOScenario(void) {}

ExampleScenario::ExampleScenario(void) : FIFOScenario()
{
    //   .    : ,     .
    setup("FIFOScenario", &FIFOScenario::init, &FIFOScenario::finish, &FIFOScenario::get_state_fifo);

    //   . NOP     .
    ADD_SCENARIO_METHOD(FIFOScenario::scen_nop);
    ADD_SCENARIO_METHOD(FIFOScenario::scen_push);
    ADD_SCENARIO_METHOD(FIFOScenario::scen_pop);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

