/** @file fifo_model.cpp
 *   FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_model.h>

namespace cpptesk {
namespace examples {
namespace fifo {

FIFO::FIFO() : iface1("input iface1"), iface2("input iface2"), iface3("output iface3"), coverage(*this)
{
    ///     .
    CPPTESK_ADD_COMPONENT(iface1);
    CPPTESK_ADD_COMPONENT(iface2);
    CPPTESK_ADD_COMPONENT(iface3);
}

FIFO::~FIFO() {}

bool FIFO::isIface1Ready() const
{
    ///  isReady  true,   iface1   - .
    return iface1.isReady();
}

bool FIFO::isIface2Ready() const
{
    ///  isReady  true,   iface2   - .
    return iface2.isReady();
}

CPPTESK_DEFINE_STIMULUS(FIFO::push_msg)
{
    //     .
    InputData data = CPPTESK_CAST_MESSAGE(InputData);

    //   FIFO.
    push_item(data.get_data());

    //       iface    .
    START_STIMULUS(PARALLEL);

    //    1  .
    CYCLE();

    //  .
    STOP();
}

CPPTESK_DEFINE_STIMULUS(FIFO::pop_msg)
{
    //     .
    InputData data = CPPTESK_CAST_MESSAGE(InputData);

    //   .
    START();

    /*      iface
       ( ,     iface  data).
     */
    RECV_STIMULUS(PARALLEL, iface, data);

    //      ,   DUT.
    OutputData outdata = OutputData(pop_item());

    //   " ,    ".
    get_pop_msg(process, iface3, outdata);

    //  .
    STOP();
}

CPPTESK_DEFINE_REACTION(FIFO::get_pop_msg)
{
    //    .
    SEND_REACTION(SEQUENTIAL, iface, message);
}

void FIFO::push_item(uint8_t data)
{
    //    ,     .
    assert(fifo.size() < 256);
    //     .
    fifo.push_back(data);
}

uint8_t FIFO::pop_item(void)
{
    //     ,    .
    assert(fifo.size() > 0);
    //     .
    uint8_t data = fifo[0];
    //     .
    fifo.erase(fifo.begin());
    //  .
    return data;
}

bool FIFO::is_empty(void)
{
    return fifo.size() == 0;
}

bool FIFO::is_full(void)
{
    return fifo.size() == 256;
}

int FIFO::fifo_fulness(void)
{
    return fifo.size();
}

void FIFO::refresh_coverage(void)
{
    coverage.out_cov_FF();
}

}}}

