/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Error types in hardware components
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <assert.h>
#include <string>

////////////////////////////////////////////////////////////////////////////////
// Strings for Different Error Types
////////////////////////////////////////////////////////////////////////////////
#define CPPTESK_NORMAL_REACTION_STRING         "[NORMAL]"
#define CPPTESK_MISSING_REACTION_STRING        "[MISSING]"
#define CPPTESK_INCORRECT_REACTION_STRING      "[INCORRECT]"
#define CPPTESK_UNEXPECTED_REACTION_STRING     "[UNEXPECTED]"
#define CPPTESK_ASSERTION_VIOLATION_STRING     "[ASSERTION]"

namespace cpptesk {
namespace hw {

enum error_type_t
{
    /// Normal reaction (no errors).
    NORMAL_REACTION      = 0,
    /// Missing reaction.
    MISSING_REACTION     = 1,
    /// Incorrect reaction.
    INCORRECT_REACTION   = 2,
    /// Unexpected reaction.
    UNEXPECTED_REACTION  = 3,
    /// Assertion violation.
    ASSERTION_VIOLATION  = 4,
    /// For internal use only.
    __ERROR_TYPE_NUMBER  = 5
};

inline std::string getErrorString(error_type_t error)
{
    switch(error)
    {
    case NORMAL_REACTION:
        return CPPTESK_NORMAL_REACTION_STRING;
    case MISSING_REACTION:
        return CPPTESK_MISSING_REACTION_STRING;
    case INCORRECT_REACTION:
        return CPPTESK_INCORRECT_REACTION_STRING;
    case UNEXPECTED_REACTION:
        return CPPTESK_UNEXPECTED_REACTION_STRING;
    case ASSERTION_VIOLATION:
        return CPPTESK_ASSERTION_VIOLATION_STRING;
    default:
        assert(false);
    }
}

}} // namespace cpptesk::hw

