/** @file fifo_media.cpp
 * CPPTESK mediator of FIFO's test system.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

#include <iostream>

namespace cpptesk {
namespace examples {
namespace fifo {

FIFOMediator::FIFOMediator()
{
    //   ,   .
    SET_INPUT_ADAPTER(iface1, FIFOMediator::serialize_iface1);
    SET_INPUT_ADAPTER(iface2, FIFOMediator::serialize_iface2);
    SET_OUTPUT_ADAPTER(iface3, FIFOMediator::deserialize_iface3);

    //     .
    SET_REACTION_TIMEOUT(FIFOMediator::REACTION_TIMEOUT);

    //  ,      .
    clear_inputs(&inputs);
    clear_outputs(&outputs);

    //     iface3.
    SET_ARBITER(iface3, matcher_iface3);

    //        .
    CALL_OUTPUT_LISTENER(FIFOMediator::listen_iface3, iface3);
    debug(DEBUG_ERROR);
}

FIFOMediator::~FIFOMediator() {}

bool FIFOMediator::isIface1Ready() const
{
    return FIFO::isIface1Ready();
}
bool FIFOMediator::isIface2Ready() const
{
    return FIFO::isIface2Ready();
}

DEFINE_PROCESS(FIFOMediator::serialize_iface1)
{
    //     .
    InputData msg = CAST_MESSAGE(InputData);

    //   .
    START_PROCESS();

#ifdef DEBUG
	std::cout << "FIFOMediator::serialize_iface1" << std::endl;
#endif

    //   iface1.
    CAPTURE_IFACE();

    //       .
    inputs.CE_WR = 1;
    inputs.DI = msg.get_data();

    //   iface1.
    RELEASE_IFACE();

    //   .
    CYCLE();

    //   .
    STOP_PROCESS();
}

DEFINE_PROCESS(FIFOMediator::serialize_iface2)
{
    //     .
    InputData msg = CAST_MESSAGE(InputData);

    //   .
    START_PROCESS();

#ifdef DEBUG
	std::cout << "FIFOMediator::serialize_iface2" << std::endl;
#endif

    //   iface2.
    CAPTURE_IFACE();

    //       .
    inputs.CE_RD = 1;

    //   iface1.
    RELEASE_IFACE();

    //   .
    CYCLE();

    //   .
    STOP_PROCESS();
}

DEFINE_PROCESS(FIFOMediator::deserialize_iface3)
{
    //     .
    OutputData &msg = CAST_MESSAGE(OutputData);

    //   .
    START_PROCESS();

#ifdef DEBUG
        std::cout << "FIFOMediator::deserialize_iface3" << std::endl;
#endif

    //     ,    .
    WAIT_REACTION(outputs.DO_VAL);

    //       .
    msg.set_data(outputs.DO);

    //     ,   .
    NEXT_REACTION();

    //   .
    STOP_PROCESS();
}

void FIFOMediator::initialize()
{
    ::start_testbench();
}

void FIFOMediator::finalize()
{
    ::stop_testbench();
}

void FIFOMediator::setInputs()
{
    ::map_state_down(&inputs);
}

void FIFOMediator::getOutputs()
{
    ::map_state_up(&outputs);
}

void FIFOMediator::simulate()
{
    ::wait_for_check();
}

}}}

